import java.sql.*;
import java.io.*;

public class CheckInputFormat{

	static final int MulTypeSinAns = 1;
	static final int MulTypeMulAns = 2;
	static final int RanType = 3;
	static final int SubType = 4;
	static final int NoOps = 5;
	static final String FormTableName = "forms";
	static final String QuestionTableName = "question_table";
	static final String StudentTableName = "student_table";
	static final String AnswerTableName = "answer_table";
	static final int MaxSubjVal = 100;
	
//length of "question_"
	static final int VarLen = 9;

	Connection conn;
	String FormNo;
	String Value[];
	String LoginId;
	Integer Qno[];
	PrintWriter out;
	String FinalMesg;
	boolean OutputDefined = false;
	boolean ErrorOccured;

//constructors

	public CheckInputFormat(Connection c, String name[], String value[], String formno, String login){
		if(name.length != value.length)
			err("unequal array lengths. Exiting");
		else{
			conn = c;
			FormNo = formno;
			LoginId = login;
			FinalMesg = "";
			ErrorOccured = false;
			Qno = new Integer[name.length];
			Value = new String[name.length];
			for(int i=0;i<name.length;i++){
				Qno[i] = new Integer(name[i].substring(VarLen,name[i].length()));
				Value[i] = value[i];
			}
		}
	}

	public CheckInputFormat(PrintWriter o, Connection c, String name[], String value[], String formno, String login){
		if(name.length != value.length)
			err("unequal array lengths. Exiting");
		else{
			out = o;
			OutputDefined = true;
			conn = c;
			LoginId = login;
			FormNo = formno;
			FinalMesg = "";
			ErrorOccured = false;
			Qno = new Integer[name.length];
			Value = new String[name.length];
			for(int i=0;i<name.length;i++){
				Qno[i] = new Integer(name[i].substring(VarLen,name[i].length()));
				Value[i] = value[i];
			}
		}
	}


//this function access the database and checks if the input is properly formatted or not. It returns 0 if the input is valid and non-zero if it is invalid.
	public int CheckInput()
	throws SQLException{
		int result = 0;
		boolean flag = false;
		if(!CheckFlag()) result = 2;
		else{
//Determining answer_number
		int oldQno = 0,qno,type;
		for(int i=1;i<=Qno.length;i++){
			qno = Qno[i-1].intValue();
			type = GetType(qno);
			if(type == -1) {
				err ("No question with number "+qno);
				result = 1;
			}
			if(qno == oldQno){
				if(type != MulTypeMulAns){
					err("Repetition of non-mamc question.");
					result = 2;
				}
				else if(!IsAnswerOK(i-1,type)){
					err("Answer "+qno+" not in right format");
					result = 4;
				}
			}
			else if(qno < oldQno){
				err("Answers not in proper order.");
				result = 5;
			}
			else{
				for(int p=oldQno+1;p<qno;p++){
					if(GetType(p) != MulTypeMulAns){ err("Question no. "+p+" not answered");
						result = 6;
						break;
					}
				}
				if((result != 6) && (IsAnswerOK(i-1,type)) && (!flag)) result = 0;
				else result = 6;
			}
			if(result != 0) break;
			oldQno = qno;
		}
//end of else
		}
		if(OutputDefined) out.println(FinalMesg);
		else System.out.println(FinalMesg);
		return result;
	}

	boolean CheckFlag(){
		boolean res = false;
		String temp="";
		try{
			Statement stmt = conn.createStatement();
			ResultSet rset = stmt.executeQuery("select valid from "+StudentTableName+" where login_id='"+LoginId+"' and form_number='"+FormNo+"'");
			res = rset.next();
			if(res) temp = rset.getString(1);
		}catch(SQLException e){
			err("error while checking flag.");
			res = false;
		}
		if(res){
			if(!temp.equals("Y")){
				res = false;
				err("<H2>You have already filled this form.</H2>");
			}
		}
		return res;
	}

//this function returns the type of the question represented by qno in 
//the database. it returns -1 in case of an error in database access.

	private int GetType(int qno)
	throws SQLException{
		int res=0;
		try{
			Statement stmt = conn.createStatement();
			ResultSet rset = stmt.executeQuery("select question_type from "+QuestionTableName+" where form_number='"+FormNo+"' and question_number="+qno);
			while(rset.next()) res=rset.getInt(1);
			stmt.close();
		}catch(SQLException e){
			res = -1;
		}
	 	return res;
	}
//this function checks whether the value of a question is right for that 
//question or not.

	boolean IsAnswerOK(int index, int type){
		boolean res=true;
		switch(type){
			case MulTypeSinAns:
			case MulTypeMulAns:{
				try{
					Integer temp;
					if(Value[index].equals("")) temp = new Integer("0");
					else temp = new Integer(Value[index]);
					if((temp.intValue() > NoOps) || (temp.intValue() < 0)){
						res = false;
						err("Option "+temp.intValue()+" invalid");
					}
				}catch(NumberFormatException e){
					res = false;
					err("Answer to question "+Qno[index].intValue()+" not in corect format");
				}
				break;
			}
			case RanType:{
				Double val=new Double("0.0");
				double ulimit=0.0,llimit=0.0;
				try{
					Statement stmt = conn.createStatement();
					ResultSet rset = stmt.executeQuery("select ulimit,llimit from "+QuestionTableName+" where form_number='"+FormNo+"' and question_number="+Qno[index].intValue());
					while(rset.next()){
						ulimit = rset.getDouble(1);
						llimit = rset.getDouble(2);
					}
					stmt.close();
				}catch(SQLException e){
					err("Could not fetch limits for question "+Qno[index].intValue()+".");
					res = false;
				}
				try{
					if(Value[index].equals("")) val = new Double("0.0");
					else val = new Double(Value[index]);
				}catch(NumberFormatException e){
					err("input to question "+Qno[index].intValue()+" should be a number between "+llimit+" and "+ulimit);
					res = false;
				}
				if((val.doubleValue() > ulimit) || (val.doubleValue() < llimit)){
					err("input to question "+Qno[index].intValue()+" should be a number between "+llimit+" and "+ulimit);
					res = false;
				}
				break;
			}
			case SubType:{
				if(Value[index].length() > MaxSubjVal){
					res = false;
					err("Input to question "+Qno[index].intValue()+" too long.");
				}
				break;
			}
			default:{
				err("Wrong question type in input.");
				res = false;
			}
		}
		return res;
	}

	private void print(String S){
		if(!ErrorOccured) FinalMesg.concat(S);
	}
	
	private void println(String S){
		if(!ErrorOccured) FinalMesg.concat(S+"\n");
	}
	
	private void err(String S){
		if(!ErrorOccured){
			ErrorOccured = true;
			FinalMesg = S;
		}
		else FinalMesg.concat(S+"\n");
	}
	
}
				
